# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,layout_ggraph)
S3method(autograph,default)
S3method(collect_connections,default)
S3method(collect_connections,layout_tbl_graph)
S3method(collect_edges,default)
S3method(collect_edges,layout_tbl_graph)
S3method(create_layout,default)
S3method(create_layout,layout_ggraph)
S3method(create_layout,tbl_graph)
S3method(format,ggraph_geometry)
S3method(ggplot_build,ggraph)
S3method(is.na,ggraph_geometry)
S3method(layout_to_table,"function")
S3method(layout_to_table,character)
S3method(layout_to_table,data.frame)
S3method(layout_to_table,default)
S3method(layout_to_table,matrix)
S3method(makeContent,cappedpathgrob)
S3method(makeContent,parallelPath)
S3method(makeContent,textalong)
S3method(scale_type,ggraph_geometry)
S3method(vec_cast,character.ggraph_geometry)
S3method(vec_ptype2,character.ggraph_geometry)
S3method(vec_ptype2,ggraph_geometry.character)
S3method(vec_ptype2,ggraph_geometry.ggraph_geometry)
export(FacetEdges)
export(FacetGraph)
export(FacetNodes)
export(GeomAxisHive)
export(GeomEdgeBezier)
export(GeomEdgeBspline)
export(GeomEdgeParallelPath)
export(GeomEdgeParallelSegment)
export(GeomEdgePath)
export(GeomEdgePoint)
export(GeomEdgeSegment)
export(GeomEdgeSf)
export(GeomEdgeSpanPath)
export(GeomEdgeSpanSegment)
export(GeomEdgeTile)
export(GeomNodeTile)
export(StatAxisHive)
export(StatConnBundle)
export(StatConnBundle0)
export(StatConnBundle2)
export(StatEdgeArc)
export(StatEdgeArc0)
export(StatEdgeArc2)
export(StatEdgeBend)
export(StatEdgeBend0)
export(StatEdgeBend2)
export(StatEdgeBundleForce)
export(StatEdgeBundleForce0)
export(StatEdgeBundleForce2)
export(StatEdgeBundleMinimal)
export(StatEdgeBundleMinimal0)
export(StatEdgeBundleMinimal2)
export(StatEdgeBundlePath)
export(StatEdgeBundlePath0)
export(StatEdgeBundlePath2)
export(StatEdgeDensity)
export(StatEdgeDiagonal)
export(StatEdgeDiagonal0)
export(StatEdgeDiagonal2)
export(StatEdgeElbow)
export(StatEdgeElbow0)
export(StatEdgeElbow2)
export(StatEdgeFan)
export(StatEdgeFan0)
export(StatEdgeFan2)
export(StatEdgeHive)
export(StatEdgeHive0)
export(StatEdgeHive2)
export(StatEdgeLink)
export(StatEdgeLink2)
export(StatEdgeLoop)
export(StatEdgeLoop0)
export(StatEdgeParallel)
export(StatEdgeParallel0)
export(StatEdgeParallel2)
export(StatFilter)
export(StatFilterSf)
export(StatNodeArcBar)
export(StatNodeCircle)
export(StatNodeVoronoi)
export(StatReverse)
export(autograph)
export(circle)
export(collect_connections)
export(collect_edges)
export(create_layout)
export(edge_angle)
export(ellipsis)
export(facet_edges)
export(facet_graph)
export(facet_nodes)
export(geom_axis_hive)
export(geom_conn_bundle)
export(geom_conn_bundle0)
export(geom_conn_bundle2)
export(geom_edge_arc)
export(geom_edge_arc0)
export(geom_edge_arc2)
export(geom_edge_bend)
export(geom_edge_bend0)
export(geom_edge_bend2)
export(geom_edge_bundle_force)
export(geom_edge_bundle_force0)
export(geom_edge_bundle_force2)
export(geom_edge_bundle_minimal)
export(geom_edge_bundle_minimal0)
export(geom_edge_bundle_minimal2)
export(geom_edge_bundle_path)
export(geom_edge_bundle_path0)
export(geom_edge_bundle_path2)
export(geom_edge_density)
export(geom_edge_diagonal)
export(geom_edge_diagonal0)
export(geom_edge_diagonal2)
export(geom_edge_elbow)
export(geom_edge_elbow0)
export(geom_edge_elbow2)
export(geom_edge_fan)
export(geom_edge_fan0)
export(geom_edge_fan2)
export(geom_edge_hive)
export(geom_edge_hive0)
export(geom_edge_hive2)
export(geom_edge_link)
export(geom_edge_link0)
export(geom_edge_link2)
export(geom_edge_loop)
export(geom_edge_loop0)
export(geom_edge_parallel)
export(geom_edge_parallel0)
export(geom_edge_parallel2)
export(geom_edge_point)
export(geom_edge_sf)
export(geom_edge_span)
export(geom_edge_span0)
export(geom_edge_span2)
export(geom_edge_tile)
export(geom_node_arc_bar)
export(geom_node_circle)
export(geom_node_label)
export(geom_node_point)
export(geom_node_range)
export(geom_node_sf)
export(geom_node_text)
export(geom_node_tile)
export(geom_node_voronoi)
export(geometry)
export(get_con)
export(get_edges)
export(get_nodes)
export(ggraph)
export(guide_edge_colorbar)
export(guide_edge_colorsteps)
export(guide_edge_colourbar)
export(guide_edge_coloursteps)
export(guide_edge_direction)
export(is.geometry)
export(label_rect)
export(layout_to_table)
export(node_angle)
export(node_rank_fabric)
export(pack_circles)
export(qgraph)
export(rectangle)
export(scale_color_viridis)
export(scale_edge_alpha)
export(scale_edge_alpha_binned)
export(scale_edge_alpha_continuous)
export(scale_edge_alpha_discrete)
export(scale_edge_alpha_identity)
export(scale_edge_alpha_manual)
export(scale_edge_color_binned)
export(scale_edge_color_brewer)
export(scale_edge_color_continuous)
export(scale_edge_color_discrete)
export(scale_edge_color_distiller)
export(scale_edge_color_fermenter)
export(scale_edge_color_gradient)
export(scale_edge_color_gradient2)
export(scale_edge_color_gradientn)
export(scale_edge_color_grey)
export(scale_edge_color_hue)
export(scale_edge_color_identity)
export(scale_edge_color_manual)
export(scale_edge_color_steps)
export(scale_edge_color_steps2)
export(scale_edge_color_stepsn)
export(scale_edge_color_viridis)
export(scale_edge_colour_binned)
export(scale_edge_colour_brewer)
export(scale_edge_colour_continuous)
export(scale_edge_colour_discrete)
export(scale_edge_colour_distiller)
export(scale_edge_colour_fermenter)
export(scale_edge_colour_gradient)
export(scale_edge_colour_gradient2)
export(scale_edge_colour_gradientn)
export(scale_edge_colour_grey)
export(scale_edge_colour_hue)
export(scale_edge_colour_identity)
export(scale_edge_colour_manual)
export(scale_edge_colour_steps)
export(scale_edge_colour_steps2)
export(scale_edge_colour_stepsn)
export(scale_edge_colour_viridis)
export(scale_edge_fill_binned)
export(scale_edge_fill_brewer)
export(scale_edge_fill_continuous)
export(scale_edge_fill_discrete)
export(scale_edge_fill_distiller)
export(scale_edge_fill_fermenter)
export(scale_edge_fill_gradient)
export(scale_edge_fill_gradient2)
export(scale_edge_fill_gradientn)
export(scale_edge_fill_grey)
export(scale_edge_fill_hue)
export(scale_edge_fill_identity)
export(scale_edge_fill_manual)
export(scale_edge_fill_steps)
export(scale_edge_fill_steps2)
export(scale_edge_fill_stepsn)
export(scale_edge_fill_viridis)
export(scale_edge_linetype)
export(scale_edge_linetype_binned)
export(scale_edge_linetype_continuous)
export(scale_edge_linetype_discrete)
export(scale_edge_linetype_identity)
export(scale_edge_linetype_manual)
export(scale_edge_radius)
export(scale_edge_shape)
export(scale_edge_shape_binned)
export(scale_edge_shape_continuous)
export(scale_edge_shape_discrete)
export(scale_edge_shape_identity)
export(scale_edge_shape_manual)
export(scale_edge_size)
export(scale_edge_size_area)
export(scale_edge_size_binned)
export(scale_edge_size_binned_area)
export(scale_edge_size_continuous)
export(scale_edge_size_discrete)
export(scale_edge_size_identity)
export(scale_edge_size_manual)
export(scale_edge_width)
export(scale_edge_width_binned)
export(scale_edge_width_continuous)
export(scale_edge_width_discrete)
export(scale_edge_width_identity)
export(scale_edge_width_manual)
export(scale_fill_viridis)
export(scale_label_size)
export(scale_label_size_binned)
export(scale_label_size_continuous)
export(scale_label_size_discrete)
export(scale_label_size_identity)
export(scale_label_size_manual)
export(set_graph_style)
export(square)
export(th_foreground)
export(th_no_axes)
export(theme_graph)
export(unset_graph_style)
import(ggplot2)
import(rlang)
import(tidygraph)
import(vctrs)
importFrom(MASS,bandwidth.nrd)
importFrom(MASS,kde2d)
importFrom(dplyr,arrange)
importFrom(dplyr,do)
importFrom(dplyr,group_by)
importFrom(dplyr,group_by_)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,pull)
importFrom(dplyr,slice)
importFrom(dplyr,summarise)
importFrom(dplyr,top_n)
importFrom(dplyr,transmute)
importFrom(dplyr,ungroup)
importFrom(ggforce,GeomArcBar)
importFrom(ggforce,GeomBezier0)
importFrom(ggforce,GeomBspline0)
importFrom(ggforce,GeomCircle)
importFrom(ggforce,GeomShape)
importFrom(ggforce,StatArcBar)
importFrom(ggforce,StatBezier)
importFrom(ggforce,StatBezier0)
importFrom(ggforce,StatBezier2)
importFrom(ggforce,StatBspline)
importFrom(ggforce,StatBspline2)
importFrom(ggforce,StatCircle)
importFrom(ggforce,StatLink)
importFrom(ggforce,StatLink2)
importFrom(ggforce,StatVoronoiTile)
importFrom(ggforce,interpolateDataFrame)
importFrom(ggforce,radial_trans)
importFrom(ggrepel,GeomLabelRepel)
importFrom(ggrepel,GeomTextRepel)
importFrom(graphlayouts,layout_as_backbone)
importFrom(graphlayouts,layout_as_metromap)
importFrom(graphlayouts,layout_with_centrality)
importFrom(graphlayouts,layout_with_centrality_group)
importFrom(graphlayouts,layout_with_constrained_stress)
importFrom(graphlayouts,layout_with_eigen)
importFrom(graphlayouts,layout_with_fixed_coords)
importFrom(graphlayouts,layout_with_focus)
importFrom(graphlayouts,layout_with_focus_group)
importFrom(graphlayouts,layout_with_pmds)
importFrom(graphlayouts,layout_with_sparse_stress)
importFrom(graphlayouts,layout_with_stress)
importFrom(grid,convertHeight)
importFrom(grid,convertWidth)
importFrom(grid,convertX)
importFrom(grid,convertY)
importFrom(grid,gList)
importFrom(grid,gTree)
importFrom(grid,gpar)
importFrom(grid,grob)
importFrom(grid,grobHeight)
importFrom(grid,grobName)
importFrom(grid,grobTree)
importFrom(grid,grobWidth)
importFrom(grid,is.unit)
importFrom(grid,makeContent)
importFrom(grid,nullGrob)
importFrom(grid,pointsGrob)
importFrom(grid,polylineGrob)
importFrom(grid,rasterGrob)
importFrom(grid,rectGrob)
importFrom(grid,segmentsGrob)
importFrom(grid,setChildren)
importFrom(grid,textGrob)
importFrom(grid,unit)
importFrom(igraph,"%--%")
importFrom(igraph,"V<-")
importFrom(igraph,"vertex_attr<-")
importFrom(igraph,E)
importFrom(igraph,V)
importFrom(igraph,add_edges)
importFrom(igraph,add_vertices)
importFrom(igraph,adjacent_vertices)
importFrom(igraph,as.undirected)
importFrom(igraph,as_edgelist)
importFrom(igraph,bfs)
importFrom(igraph,components)
importFrom(igraph,count_components)
importFrom(igraph,degree)
importFrom(igraph,delete_edges)
importFrom(igraph,delete_vertex_attr)
importFrom(igraph,distances)
importFrom(igraph,edge_attr)
importFrom(igraph,ends)
importFrom(igraph,gorder)
importFrom(igraph,graph_attr)
importFrom(igraph,gsize)
importFrom(igraph,incident_edges)
importFrom(igraph,induced_subgraph)
importFrom(igraph,is.directed)
importFrom(igraph,is.named)
importFrom(igraph,is_directed)
importFrom(igraph,layout_as_bipartite)
importFrom(igraph,layout_as_star)
importFrom(igraph,layout_as_tree)
importFrom(igraph,layout_in_circle)
importFrom(igraph,layout_nicely)
importFrom(igraph,layout_on_grid)
importFrom(igraph,layout_on_sphere)
importFrom(igraph,layout_randomly)
importFrom(igraph,layout_with_dh)
importFrom(igraph,layout_with_drl)
importFrom(igraph,layout_with_fr)
importFrom(igraph,layout_with_gem)
importFrom(igraph,layout_with_graphopt)
importFrom(igraph,layout_with_kk)
importFrom(igraph,layout_with_lgl)
importFrom(igraph,layout_with_mds)
importFrom(igraph,layout_with_sugiyama)
importFrom(igraph,mst)
importFrom(igraph,neighbors)
importFrom(igraph,permute)
importFrom(igraph,shortest_paths)
importFrom(igraph,simplify)
importFrom(igraph,unfold_tree)
importFrom(igraph,vertex_attr)
importFrom(lifecycle,deprecated)
importFrom(memoise,memoise)
importFrom(rlang,.data)
importFrom(rlang,as_quosure)
importFrom(rlang,enquo)
importFrom(rlang,eval_tidy)
importFrom(rlang,quo_is_null)
importFrom(rlang,quo_is_symbol)
importFrom(rlang,quo_text)
importFrom(rlang,quos)
importFrom(rlang,sym)
importFrom(scales,identity_pal)
importFrom(scales,muted)
importFrom(scales,rescale_pal)
importFrom(stats,setNames)
importFrom(tidygraph,.G)
importFrom(tidygraph,.register_graph_context)
importFrom(tidygraph,activate)
importFrom(tidygraph,arrange)
importFrom(tidygraph,as_tbl_graph)
importFrom(tidygraph,graph_is_forest)
importFrom(tidygraph,graph_is_tree)
importFrom(tidygraph,node_is_root)
importFrom(utils,head)
importFrom(utils,tail)
importFrom(viridis,scale_color_viridis)
importFrom(viridis,scale_colour_viridis)
importFrom(viridis,scale_fill_viridis)
useDynLib(ggraph, .registration = TRUE)
