#' Membership network of American Whigs
#'
#' This dataset shows the membership of 136 colonial Americans in 5 whig
#' organization and is a bipartite graph. The data appeared in the appendix to
#' David Hackett Fischer's *Paul Revere's Ride* (Oxford University Press,
#' 1995) and compiled by Kieran Healy for the blog post
#' [Using Metadata to Find Paul Revere](https://kieranhealy.org/blog/archives/2013/06/09/using-metadata-to-find-paul-revere/).
#'
#' @format
#' The data is stored as an incidence matrix with persons as rows and
#' organizations as columns. A 0 means no membership while a one means
#' membership.
#'
#' @source <https://github.com/kjhealy/revere/blob/master/data/PaulRevereAppD.csv>
#' adapted from:
#'
#' Fischer, David H. (1995) *Paul Revere's Ride*. Oxford University Press
#'
'whigs'
