#' @aliases ggraph-package
'_PACKAGE'

# The following block is used by usethis to automatically manage
# roxygen namespace tags. Modify with care!
## usethis namespace: start
#' @useDynLib ggraph
#' @import ggplot2 tidygraph
#' @importFrom Rcpp sourceCpp
#' @importFrom rlang .data
## usethis namespace: end
NULL
