% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/autograph.R
\name{qgraph}
\alias{qgraph}
\title{Deprecated autograph predecessor}
\usage{
qgraph(...)
}
\description{
Deprecated autograph predecessor
}
\keyword{internal}
